/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.web.templ;

import java.util.Map;
import rx.Observable;
import rx.Single;
import io.vertx.core.AsyncResult;
import io.vertx.core.Handler;

/**
 * A template engine uses a specific template and the data in a routing context to render a resource into a buffer.
 * <p>
 * Concrete implementations exist for several well-known template engines.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.templ.TemplateEngine original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.ext.web.templ.TemplateEngine.class)
public class TemplateEngine extends io.vertx.rxjava.ext.web.common.template.TemplateEngine {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    TemplateEngine that = (TemplateEngine) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<TemplateEngine> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new TemplateEngine((io.vertx.ext.web.templ.TemplateEngine) obj),
    TemplateEngine::getDelegate
  );

  private final io.vertx.ext.web.templ.TemplateEngine delegate;
  
  public TemplateEngine(io.vertx.ext.web.templ.TemplateEngine delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public io.vertx.ext.web.templ.TemplateEngine getDelegate() {
    return delegate;
  }

  /**
   * Render the template
   * @param context the routing context
   * @param templateFileName the template file name to use
   * @param handler the handler that will be called with a result containing the buffer or a failure.
   */
  @Deprecated()
  public void render(io.vertx.rxjava.ext.web.RoutingContext context, String templateFileName, Handler<AsyncResult<io.vertx.rxjava.core.buffer.Buffer>> handler) { 
    delegate.render(context.getDelegate(), templateFileName, new Handler<AsyncResult<io.vertx.core.buffer.Buffer>>() {
      public void handle(AsyncResult<io.vertx.core.buffer.Buffer> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.core.buffer.Buffer.newInstance(ar.result())));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
  }

    /**
   * Render the template
   * @param context the routing context
   * @param templateFileName the template file name to use
   * @return 
   * @deprecated use {@link #rxRender} instead
   */
  @Deprecated()
  public Observable<io.vertx.rxjava.core.buffer.Buffer> renderObservable(io.vertx.rxjava.ext.web.RoutingContext context, String templateFileName) { 
    io.vertx.rx.java.ObservableFuture<io.vertx.rxjava.core.buffer.Buffer> handler = io.vertx.rx.java.RxHelper.observableFuture();
    render(context, templateFileName, handler.toHandler());
    return handler;
  }

  /**
   * Render the template
   * @param context the routing context
   * @param templateFileName the template file name to use
   * @return 
   */
  @Deprecated()
  public Single<io.vertx.rxjava.core.buffer.Buffer> rxRender(io.vertx.rxjava.ext.web.RoutingContext context, String templateFileName) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      render(context, templateFileName, fut);
    }));
  }

  /**
   * Render the template
   * <p>
   * <b>NOTE</b> if you call method directly (i.e. not using {@link io.vertx.rxjava.ext.web.handler.TemplateHandler}) make sure
   * that <i>templateFileName</i> is sanitized via 
   * @param context the routing context
   * @param templateDirectory the template directory to use
   * @param templateFileName the relative template file name to use
   * @param handler the handler that will be called with a result containing the buffer or a failure.
   */
  @Deprecated()
  public void render(io.vertx.rxjava.ext.web.RoutingContext context, String templateDirectory, String templateFileName, Handler<AsyncResult<io.vertx.rxjava.core.buffer.Buffer>> handler) { 
    delegate.render(context.getDelegate(), templateDirectory, templateFileName, new Handler<AsyncResult<io.vertx.core.buffer.Buffer>>() {
      public void handle(AsyncResult<io.vertx.core.buffer.Buffer> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.core.buffer.Buffer.newInstance(ar.result())));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
  }

    /**
   * Render the template
   * <p>
   * <b>NOTE</b> if you call method directly (i.e. not using {@link io.vertx.rxjava.ext.web.handler.TemplateHandler}) make sure
   * that <i>templateFileName</i> is sanitized via 
   * @param context the routing context
   * @param templateDirectory the template directory to use
   * @param templateFileName the relative template file name to use
   * @return 
   * @deprecated use {@link #rxRender} instead
   */
  @Deprecated()
  public Observable<io.vertx.rxjava.core.buffer.Buffer> renderObservable(io.vertx.rxjava.ext.web.RoutingContext context, String templateDirectory, String templateFileName) { 
    io.vertx.rx.java.ObservableFuture<io.vertx.rxjava.core.buffer.Buffer> handler = io.vertx.rx.java.RxHelper.observableFuture();
    render(context, templateDirectory, templateFileName, handler.toHandler());
    return handler;
  }

  /**
   * Render the template
   * <p>
   * <b>NOTE</b> if you call method directly (i.e. not using {@link io.vertx.rxjava.ext.web.handler.TemplateHandler}) make sure
   * that <i>templateFileName</i> is sanitized via 
   * @param context the routing context
   * @param templateDirectory the template directory to use
   * @param templateFileName the relative template file name to use
   * @return 
   */
  @Deprecated()
  public Single<io.vertx.rxjava.core.buffer.Buffer> rxRender(io.vertx.rxjava.ext.web.RoutingContext context, String templateDirectory, String templateFileName) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      render(context, templateDirectory, templateFileName, fut);
    }));
  }

  /**
   * Returns true if the template engine caches template files. If false, then template files are freshly loaded each
   * time they are used.
   * @return True if template files are cached; otherwise, false.
   */
  public boolean isCachingEnabled() { 
    boolean ret = delegate.isCachingEnabled();
    return ret;
  }


  public static  TemplateEngine newInstance(io.vertx.ext.web.templ.TemplateEngine arg) {
    return arg != null ? new TemplateEngine(arg) : null;
  }
}
