/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.pgclient;

import java.util.Map;
import rx.Observable;
import rx.Single;
import java.util.List;
import io.vertx.pgclient.PgConnectOptions;
import io.vertx.core.AsyncResult;
import io.vertx.core.Handler;
import io.vertx.sqlclient.PoolOptions;

/**
 * A pool of PostgreSQL connections.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.pgclient.PgPool original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.pgclient.PgPool.class)
public class PgPool extends io.vertx.rxjava.sqlclient.Pool {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    PgPool that = (PgPool) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<PgPool> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new PgPool((io.vertx.pgclient.PgPool) obj),
    PgPool::getDelegate
  );

  private final io.vertx.pgclient.PgPool delegate;
  
  public PgPool(io.vertx.pgclient.PgPool delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public io.vertx.pgclient.PgPool getDelegate() {
    return delegate;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with a default <code>poolOptions</code>.
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool() { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool());
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with <code>connectOptions</code> build from the environment variables.
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with a default <code>poolOptions</code>.
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(String connectionUri) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with <code>connectOptions</code> build from <code>connectionUri</code>.
   * @param connectionUri 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(String connectionUri, PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(connectionUri, poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with a default <code>poolOptions</code>.
   * @param vertx 
   * @param connectionUri 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(io.vertx.rxjava.core.Vertx vertx, String connectionUri) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(vertx.getDelegate(), connectionUri));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with <code>connectOptions</code> build from the environment variables.
   * @param vertx 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(io.vertx.rxjava.core.Vertx vertx, PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(vertx.getDelegate(), poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with <code>connectOptions</code> build from <code>connectionUri</code>.
   * @param vertx 
   * @param connectionUri 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(io.vertx.rxjava.core.Vertx vertx, String connectionUri, PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(vertx.getDelegate(), connectionUri, poolOptions));
    return ret;
  }

  /**
   * Create a connection pool to the database configured with the given <code>connectOptions</code> and <code>poolOptions</code>.
   * @param connectOptions 
   * @param poolOptions the options for creating the pool
   * @return the connection pool
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(PgConnectOptions connectOptions, PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(connectOptions, poolOptions));
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.pgclient.PgPool#pool} with a specific  instance.
   * @param vertx 
   * @param connectOptions 
   * @param poolOptions 
   * @return 
   */
  public static io.vertx.rxjava.pgclient.PgPool pool(io.vertx.rxjava.core.Vertx vertx, PgConnectOptions connectOptions, PoolOptions poolOptions) { 
    io.vertx.rxjava.pgclient.PgPool ret = io.vertx.rxjava.pgclient.PgPool.newInstance(io.vertx.pgclient.PgPool.pool(vertx.getDelegate(), connectOptions, poolOptions));
    return ret;
  }

  public io.vertx.rxjava.pgclient.PgPool preparedQuery(String sql, Handler<AsyncResult<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>>> handler) { 
    delegate.preparedQuery(sql, new Handler<AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>>>() {
      public void handle(AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.sqlclient.RowSet.newInstance(ar.result(), (io.vertx.lang.rx.TypeArg)io.vertx.rxjava.sqlclient.Row.__TYPE_ARG)));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    public Single<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>> rxPreparedQuery(String sql) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      preparedQuery(sql, fut);
    }));
  }

  public io.vertx.rxjava.pgclient.PgPool query(String sql, Handler<AsyncResult<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>>> handler) { 
    delegate.query(sql, new Handler<AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>>>() {
      public void handle(AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.sqlclient.RowSet.newInstance(ar.result(), (io.vertx.lang.rx.TypeArg)io.vertx.rxjava.sqlclient.Row.__TYPE_ARG)));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    public Single<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>> rxQuery(String sql) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      query(sql, fut);
    }));
  }

  public io.vertx.rxjava.pgclient.PgPool preparedQuery(String sql, io.vertx.rxjava.sqlclient.Tuple arguments, Handler<AsyncResult<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>>> handler) { 
    delegate.preparedQuery(sql, arguments.getDelegate(), new Handler<AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>>>() {
      public void handle(AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.sqlclient.RowSet.newInstance(ar.result(), (io.vertx.lang.rx.TypeArg)io.vertx.rxjava.sqlclient.Row.__TYPE_ARG)));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    public Single<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>> rxPreparedQuery(String sql, io.vertx.rxjava.sqlclient.Tuple arguments) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      preparedQuery(sql, arguments, fut);
    }));
  }

  public io.vertx.rxjava.pgclient.PgPool preparedBatch(String sql, List<io.vertx.rxjava.sqlclient.Tuple> batch, Handler<AsyncResult<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>>> handler) { 
    delegate.preparedBatch(sql, batch.stream().map(elt -> elt.getDelegate()).collect(java.util.stream.Collectors.toList()), new Handler<AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>>>() {
      public void handle(AsyncResult<io.vertx.sqlclient.RowSet<io.vertx.sqlclient.Row>> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture(io.vertx.rxjava.sqlclient.RowSet.newInstance(ar.result(), (io.vertx.lang.rx.TypeArg)io.vertx.rxjava.sqlclient.Row.__TYPE_ARG)));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    public Single<io.vertx.rxjava.sqlclient.RowSet<io.vertx.rxjava.sqlclient.Row>> rxPreparedBatch(String sql, List<io.vertx.rxjava.sqlclient.Tuple> batch) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      preparedBatch(sql, batch, fut);
    }));
  }


  public static  PgPool newInstance(io.vertx.pgclient.PgPool arg) {
    return arg != null ? new PgPool(arg) : null;
  }
}
